Package: seqsetvis
Type: Package
Title: Set Based Visualizations for Next-Gen Sequencing Data
Version: 1.32.0
Authors@R: person("Joseph R", "Boyd", email = "jrboyd@uvm.edu",
                  role = c("aut", "cre"), comment = c(ORCID="0000-0002-8969-9676"))
Description: seqsetvis enables the visualization and analysis of sets
        of genomic sites in next gen sequencing data. Although
        seqsetvis was designed for the comparison of mulitple ChIP-seq
        samples, this package is domain-agnostic and allows the
        processing of multiple genomic coordinate files (bed-like
        files) and signal files (bigwig files pileups from bam file).
        seqsetvis has multiple functions for fetching data from regions
        into a tidy format for analysis in data.table or tidyverse and
        visualization via ggplot2.
License: MIT + file LICENSE
Encoding: UTF-8
Suggests: BiocFileCache, BiocManager, BiocStyle, ChIPpeakAnno,
        GenomeInfoDb, covr, knitr, rmarkdown, testthat
Depends: R (>= 4.3), ggplot2
Imports: cowplot, data.table, eulerr, Seqinfo, GenomicAlignments,
        GenomicRanges, ggplotify, grDevices, grid, IRanges, limma,
        methods, pbapply, pbmcapply, png, RColorBrewer, Rsamtools,
        rtracklayer, S4Vectors, scales, stats, UpSetR
RoxygenNote: 7.3.3
Roxygen: list(markdown = TRUE)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-30 06:29:48 UTC; root
biocViews: Software, ChIPSeq, MultipleComparison, Sequencing,
        Visualization
Config/pak/sysreqs: cmake make libbz2-dev liblzma-dev libpng-dev
        libuv1-dev libxml2-dev libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:47:37 UTC
RemoteUrl: https://github.com/bioc/seqsetvis
RemoteRef: RELEASE_3_23
RemoteSha: 5c54cba0c5280d10b12fa2b088b441ada81b793c
Author: Joseph R Boyd [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8969-9676>)
Maintainer: Joseph R Boyd <jrboyd@uvm.edu>
Built: R 4.6.0; ; 2026-04-30 07:22:52 UTC; unix
