# Generated by roxygen2: do not edit by hand

export(add_cluster_annotation)
export(append_ynorm)
export(applyMovingAverage)
export(applySpline)
export(assemble_heatmap_cluster_bars)
export(calc_norm_factors)
export(centerAtMax)
export(centerFixedSizeGRanges)
export(centerGRangesAtMax)
export(clusteringKmeans)
export(clusteringKmeansNestedHclust)
export(col2hex)
export(collapse_gr)
export(convert_collapsed_coord)
export(copy_clust_info)
export(crossCorrByRle)
export(easyLoad_FUN)
export(easyLoad_IDRmerged)
export(easyLoad_bed)
export(easyLoad_broadPeak)
export(easyLoad_narrowPeak)
export(easyLoad_seacr)
export(expandCigar)
export(findMaxPos)
export(fragLen_calcStranded)
export(fragLen_fromMacs2Xls)
export(getReadLength)
export(get_mapped_reads)
export(ggellipse)
export(harmonize_seqlengths)
export(make_clustering_matrix)
export(merge_clusters)
export(prepare_fetch_GRanges)
export(prepare_fetch_GRanges_names)
export(prepare_fetch_GRanges_width)
export(quantileGRangesWidth)
export(reorder_clusters_hclust)
export(reorder_clusters_manual)
export(reorder_clusters_stepdown)
export(reverse_clusters)
export(safeBrew)
export(split_cluster)
export(ssvAnnotateSubjectGRanges)
export(ssvConsensusIntervalSets)
export(ssvFactorizeMembTable)
export(ssvFeatureBars)
export(ssvFeatureBinaryHeatmap)
export(ssvFeatureEuler)
export(ssvFeaturePie)
export(ssvFeatureUpset)
export(ssvFeatureVenn)
export(ssvFetchBam)
export(ssvFetchBamPE)
export(ssvFetchBamPE.RNA)
export(ssvFetchBigwig)
export(ssvFetchGRanges)
export(ssvFetchSignal)
export(ssvMakeMembTable)
export(ssvOverlapIntervalSets)
export(ssvSignalBandedQuantiles)
export(ssvSignalClustering)
export(ssvSignalHeatmap)
export(ssvSignalHeatmap.ClusterBars)
export(ssvSignalLineplot)
export(ssvSignalLineplotAgg)
export(ssvSignalScatterplot)
export(viewGRangesWinSample_dt)
export(viewGRangesWinSummary_dt)
export(within_clust_sort)
exportMethods(ssvAnnotateSubjectGRanges)
exportMethods(ssvMakeMembTable)
import(GenomicAlignments)
import(GenomicRanges)
import(Rsamtools)
import(S4Vectors)
import(Seqinfo)
import(UpSetR)
import(cowplot)
import(data.table, except = c(shift, first, second, last))
import(eulerr)
import(ggplot2)
import(ggplotify)
import(methods)
import(pbapply)
import(pbmcapply)
import(png)
import(rtracklayer)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,scanBamHeader)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(grDevices,col2rgb)
importFrom(grDevices,gray)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grid,rasterGrob)
importFrom(limma,vennCounts)
importFrom(pbapply,pblapply)
importFrom(pbmcapply,pbmclapply)
importFrom(scales,hue_pal)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,quantile)
importFrom(stats,spline)
importFrom(stats,weighted.mean)
