#' easy awesome peak set vis
#' TESTING
#' seqsetvis allows you to...
#'
#' 2 steps
#' \code{\link{ssvOverlapIntervalSets}}.
#' \code{\link{ssvFetchBigwig}}.
#' Otherwise refer to the vignettes to see
"_PACKAGE"
