% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions_feature_plots.R
\name{ssvFeaturePie}
\alias{ssvFeaturePie}
\title{ssvFeaturePie}
\usage{
ssvFeaturePie(object, slice_colors = NULL, return_data = FALSE)
}
\arguments{
\item{object}{object that ssvMakeMembTable can convert to logical matrix
membership}

\item{slice_colors}{colors to use for pie slices}

\item{return_data}{logical.  If TRUE, return value is no longer ggplot and is
instead the data used to generate that plot. Default is FALSE.}
}
\value{
ggplot pie graph of set sizes
}
\description{
Generate a ggplot pie plot of set sizes.
}
\examples{
data(CTCF_in_10a_overlaps_gr)
ssvFeaturePie(list(1:3, 2:6))
ssvFeaturePie(CTCF_in_10a_overlaps_gr)
ssvFeaturePie(S4Vectors::mcols(CTCF_in_10a_overlaps_gr)[,2:3])
}
